library ieee;
use ieee.std_logic_1164.all;
use work.pack_2.all;

entity uut is
  port (x_fghij : in  X01_vector(20 downto 13);
        z_abcde : out X01_vector(20 downto 13));
end uut;

architecture structure of uut is
  component sn7400 -- 2er-nand
    port (x : in  X01_vector (1 to 2);
          y : out X01);
  end component;
  component sn7404 is -- not
    port (x : in X01; y : out X01);
  end component;
  component sn7472 is -- JK-MS-Flipflop
    port (s_b,r_b,c : in X01; j,k : in X01_vector(1 to 3); q,q_b : out X01);
  end component;

  alias c : x01 is x_fghij(15);
  alias reset : x01 is x_fghij(20);
  alias load : x01 is x_fghij(19);
  alias d1 : x01 is x_fghij(18);
  alias d0 : x01 is x_fghij(17);
  alias count : x01 is x_fghij(14);
  alias compl : x01 is x_fghij(13);
  
  alias q1: x01 is z_abcde(20); -- ff1 ergebnis
  alias q0: x01 is z_abcde(19); -- ff0 ergebnis
  
  signal nreset, ncompl, ncount: X01; -- not reset, not compl, not count
  signal ns1, ns0 : X01; -- ergebnis fr stat. s eingnge
  signal nr1, nr0 : X01; -- ergebnis fr stat. r eingnge
  signal r1, r0 : X01; -- zwischenergebis fr stat. r eingnge
  signal jk1, jk0 : X01; -- zwischenergebnis fr dyn. jk-eingnge
  signal sig11, sig12 : X01; -- zwischenergebnisse fr 1. ff (Q1)
  signal sig01, sig02 : X01; -- zwischenergebnisse fr 2. ff (Q0)
  signal z_q0 : X01; -- ergebnis 2. ff (Q0)
begin
  -- not: reset, compl => nreset, ncompl
  b11: sn7404 port map (x=>reset, y=>nreset);
  b12: sn7404 port map (x=>compl, y=>ncompl);
  
  -- not s1 <= load nand d1
  b13: sn7400 port map (x(1)=>load, x(2)=>d1, y=>ns1);
  
  -- not r1 <= not ((ns1 nand load) nand nreset)
  b14: sn7400 port map (x(1)=>ns1, x(2)=>load, y=>sig11);
  b15: sn7400 port map (x(1)=>sig11, x(2)=>nreset, y=>r1);
  b16: sn7404 port map (x=>r1, y=>nr1);
  
  -- j1=k1 <= (count nand q0) nand ncompl
  b17: sn7400 port map (x(1)=>count, x(2)=>z_q0, y=>sig12);
  b18: sn7400 port map (x(1)=>sig12, x(2)=>ncompl, y=>jk1);
  
  -- not s0 <= load nand d0
  b21: sn7400 port map (x(1)=>load, x(2)=>d0, y=>ns0);
  
  -- not r0 <= not ((ns0 nand load) nand nreset)
  b22: sn7400 port map (x(1)=>ns0, x(2)=>load, y=>sig01);
  b23: sn7400 port map (x(1)=>sig01, x(2)=>nreset, y=>r0);
  b24: sn7404 port map (x=>r0, y=>nr0);
  
  -- j0=k0 <= (not count nand ncompl)
  b25: sn7404 port map (x=>count, y=>ncount);
  b26: sn7400 port map (x(1)=>ncount, x(2)=>ncompl, y=>jk0);

  -- 1. ff (q1)
  ff1: sn7472 port map (s_b=>ns1, r_b=>nr1, c=>c,
                        j(1)=>jk1, j(2)=>'1', j(3)=>'1',
                        k(1)=>jk1, k(2)=>'1', k(3)=>'1',
                        q=>q1);
                        
  -- 2. ff (q0)
  ff2: sn7472 port map (s_b=>ns0, r_b=>nr0, c=>c,
                        j(1)=>jk0, j(2)=>'1', j(3)=>'1',
                        k(1)=>jk0, k(2)=>'1', k(3)=>'1',
                        q=>z_q0);
  q0 <= z_q0;
end structure;
